@@ -81,7 +81,7 @@ PART_TO_CLASSES = {
         "basic_set_list", "set_list", "aff_list", "pw_aff_list", "band_list",
         "printer",  "mat", "vec", "id",
         "aff", "pw_aff",
-        "multi_aff", "pw_multi_aff",
+        "multi_aff", "multi_pw_aff", "pw_multi_aff", "union_pw_multi_aff",
 
          "constraint", "space", "local_space",
          ],
@@ -101,7 +101,7 @@ PART_TO_CLASSES = {
 
         "band", "schedule",
 
-        "access_info", "flow",
+        "access_info", "flow", "restriction",
         ]
         }
 CLASSES = []
@@ -111,6 +111,7 @@ for cls_list in PART_TO_CLASSES.itervalues():
 CLASS_MAP = {
         "equality": "constraint",
         "inequality": "constraint",
+        "options": "ctx",
         }
 
 ENUMS = ["isl_dim_type", "isl_fold"]
@@ -342,7 +343,9 @@ class FunctionData:
 
     def parse_decl(self, decl):
         decl_match = DECL_RE.match(decl)
-        assert decl_match is not None, decl
+        if decl_match is None:
+            print "WARNING: func decl regexp not matched: %s" % decl
+            return
 
         return_base_type = decl_match.group(1)
         return_ptr = decl_match.group(2)
@@ -350,7 +353,10 @@ class FunctionData:
         args = [i.strip()
                 for i in split_at_unparenthesized_commas(decl_match.group(4))]
 
-        assert c_name.startswith("isl_")
+        if c_name == "ISL_ARG_DECL":
+            return
+
+        assert c_name.startswith("isl_"), c_name
         name = c_name[4:]
 
         found_class = False
@@ -382,6 +388,10 @@ class FunctionData:
         if name in PYTHON_RESERVED_WORDS:
             name = name + "_"
 
+        if cls == "options":
+            assert name.startswith("set_") or name.startswith("get_")
+            name = name[:4]+"option_"+name[4:]
+
         words = return_base_type.split()
 
         is_exported = "__isl_export" in words
@@ -504,7 +514,6 @@ def write_wrapper(outf, meth):
             assert meth.args[arg_idx].name == "user"
 
             cb_name = "cb_%s_%s_%s" % (meth.cls, meth.name, arg.name)
-            py_cb_name = "py_"+arg.name
 
             input_args.append("py::object py_%s" % arg.name)
             passed_args.append(cb_name)
@@ -798,7 +807,7 @@ def write_wrapper(outf, meth):
             ret_descr = "None"
 
     else:
-        raise NotImplementedError("ret type: %s %s in %s" % (
+        raise SignatureNotSupported("ret type: %s %s in %s" % (
             meth.return_base_type, meth.return_ptr, meth))
 
     outf.write("""
@@ -910,6 +919,7 @@ def gen_wrapper(include_dirs):
     fdata.read_header("isl/band.h")
     fdata.read_header("isl/schedule.h")
     fdata.read_header("isl/flow.h")
+    fdata.read_header("isl/options.h")
 
 
     for part, classes in PART_TO_CLASSES.iteritems():
