@@ -5,13 +5,15 @@ from islpy.version import *
 _CHECK_DIM_TYPES = [
         dim_type.in_, dim_type.param, dim_type.set]
 
+ALL_CLASSES = tuple(getattr(_isl, cls) for cls in dir(_isl) if cls[0].isupper())
+EXPR_CLASSES = tuple(cls for cls in ALL_CLASSES
+        if "Aff" in cls.__name__ or "Polynomial" in cls.__name__)
+
 _DEFAULT_CONTEXT = Context()
 
 def _add_functionality():
     import islpy._isl as _isl
 
-    ALL_CLASSES = [getattr(_isl, cls) for cls in dir(_isl) if cls[0].isupper()]
-
     # {{{ generic initialization
 
     def obj_new_from_string(cls, s, context=None):
@@ -145,9 +147,9 @@ def _add_functionality():
     def space_create_from_names(ctx, set=None, in_=None, out=None, params=[]):
         """Create a :class:`Space` from lists of variable names.
 
-        :param set_`: names of `set`-type variables.
-        :param in_`: names of `in`-type variables.
-        :param out`: names of `out`-type variables.
+        :param set_: names of `set`-type variables.
+        :param in_: names of `in`-type variables.
+        :param out: names of `out`-type variables.
         :param params`: names of parameter-type variables.
         """
         dt = dim_type
@@ -288,7 +290,7 @@ def _add_functionality():
         """Create a constraint `const + coeff_1*var_1 +... >= 0`.
 
         :param space: :class:`Space`
-        :param coefficients: a :class:`dict` or iterable of :class:`tuple` 
+        :param coefficients: a :class:`dict` or iterable of :class:`tuple`
             instances mapping variable names to their coefficients
             The constant is set to the value of the key '1'.
 
@@ -345,6 +347,29 @@ def _add_functionality():
 
     # }}}
 
+    # {{{ common functionality
+
+    def obj_get_var_dict(self, dimtype=None):
+        """Return a dictionary mapping variable names to tuples of (:class:`dim_type`, index).
+
+        :param dimtype: None to get all variables, otherwise
+            one of :class:`dim_type`.
+        """
+        return self.get_space().get_var_dict(dimtype)
+
+    def obj_get_var_names(self, dimtype):
+        """Return a list of dim names (in order) for :class:`dim_type` *dimtype*."""
+        return [self.get_dim_name(dimtype, i) for i in xrange(self.dim(dimtype))]
+
+    for cls in ALL_CLASSES:
+        if hasattr(cls, "get_space") and cls is not Space:
+            cls.get_var_dict = obj_get_var_dict
+            cls.get_var_names = obj_get_var_names
+            cls.space = property(cls.get_space)
+
+    # }}}
+
+
     # {{{ PwAff
 
     def pwaff_get_pieces(self):
@@ -384,7 +409,7 @@ def _add_functionality():
         if isinstance(template, PwAff):
             result = PwAff.empty(template.get_space())
             for set, _ in template.get_pieces():
-                result = result.cond(set, number_aff, result)
+                result = set.indicator_function().cond(number_aff, result)
             return result
 
         else:
@@ -423,6 +448,9 @@ def _add_functionality():
         except TypeError:
             return NotImplemented
 
+    def aff_flordiv(self, other):
+        return self.scale_down(other).floor()
+
     for aff_class in [Aff, PwAff]:
         aff_class.__add__ = aff_add
         aff_class.__radd__ = aff_add
@@ -432,6 +460,7 @@ def _add_functionality():
         aff_class.__rmul__ = aff_mul
         aff_class.__neg__ = aff_class.neg
         aff_class.__mod__ = aff_class.mod
+        aff_class.__floordiv__ = aff_flordiv
 
     # }}}
 
@@ -450,6 +479,7 @@ def _add_functionality():
 
         from inspect import ismethod
         for method_name in dir(special_class):
+            # do not overwrite existing methods
             if hasattr(basic_class, method_name):
                 continue
 
@@ -584,7 +614,61 @@ _add_functionality()
 
 
 
-def align_spaces(obj, tgt, obj_bigger_ok=False):
+def _align_dim_type(tgt_dt, obj, tgt, obj_bigger_ok, obj_names, tgt_names):
+    if None in tgt_names:
+        all_nones = [None] * len(tgt_names)
+        if tgt_names == all_nones and obj_names == all_nones:
+            # that's ok
+            return obj
+
+        raise RuntimeError("tgt may not contain any unnamed dimensions")
+
+    obj_names = set(obj_names) - set([None])
+    tgt_names = set(tgt_names) - set([None])
+
+    names_in_both = obj_names & tgt_names
+
+    tgt_idx = 0
+    while tgt_idx < tgt.dim(tgt_dt):
+        tgt_name = tgt.get_dim_name(tgt_dt, tgt_idx)
+
+        if tgt_name in names_in_both:
+            if (obj.dim(tgt_dt) > tgt_idx
+                    and tgt_name == obj.get_dim_name(tgt_dt, tgt_idx)):
+                tgt_idx += 1
+            else:
+                src_dt, src_idx = obj.get_var_dict()[tgt_name]
+
+                if src_dt == tgt_dt:
+                    assert src_idx > tgt_idx
+
+                    # isl requires move_dims to be between different types.
+                    # Not sure why. Let's make it happy.
+                    other_dt = dim_type.param
+                    if src_dt == other_dt:
+                        other_dt = dim_type.out
+
+                    other_dt_dim = obj.dim(other_dt)
+                    obj = obj.move_dims(other_dt, other_dt_dim, src_dt, src_idx, 1)
+                    obj = obj.move_dims(tgt_dt, tgt_idx, other_dt, other_dt_dim, 1)
+                else:
+                    obj = obj.move_dims(tgt_dt, tgt_idx, src_dt, src_idx, 1)
+
+                tgt_idx += 1
+        else:
+            obj = obj.insert_dims(tgt_dt, tgt_idx, 1)
+            obj = obj.set_dim_name(tgt_dt, tgt_idx, tgt_name)
+            tgt_idx += 1
+
+    if tgt_idx < obj.dim(tgt_dt) and not obj_bigger_ok:
+        raise ValueError("obj has leftover dimensions")
+
+    return obj
+
+
+
+
+def align_spaces(obj, tgt, obj_bigger_ok=False, across_dim_types=False):
     """
     Try to make the space in which *obj* lives the same as that of *tgt* by
     adding/matching named dimensions.
@@ -592,54 +676,51 @@ def align_spaces(obj, tgt, obj_bigger_ok=False):
     :param obj_bigger_ok: If *True*, no error is raised if the resulting *obj*
         has more dimensions than *tgt*.
     """
-    for dt in _CHECK_DIM_TYPES:
-        obj_names = [obj.get_dim_name(dt, i) for i in xrange(obj.dim(dt))]
-        tgt_names = [tgt.get_dim_name(dt, i) for i in xrange(tgt.dim(dt))]
 
-        if None in tgt_names:
-            all_nones = [None] * len(tgt_names)
-            if tgt_names == all_nones and obj_names == all_nones:
-                # that's ok
-                continue
+    if isinstance(tgt, EXPR_CLASSES):
+        dim_types = _CHECK_DIM_TYPES[:]
+        dim_types.remove(dim_type.out)
+    else:
+        dim_types = _CHECK_DIM_TYPES
 
-            raise RuntimeError("tgt may not contain any unnamed dimensions")
+    if across_dim_types:
+        obj_names = [obj.get_dim_name(dt, i)
+                for dt in dim_types
+                for i in xrange(obj.dim(dt))
+                ]
+        tgt_names = [tgt.get_dim_name(dt, i)
+                for dt in dim_types
+                for i in xrange(tgt.dim(dt))
+                ]
 
-        obj_names = set(obj_names) - set([None])
-        tgt_names = set(tgt_names) - set([None])
+        for dt in dim_types:
+            obj = _align_dim_type(dt, obj, tgt, obj_bigger_ok, obj_names, tgt_names)
+    else:
+        for dt in dim_types:
+            obj_names = [obj.get_dim_name(dt, i) for i in xrange(obj.dim(dt))]
+            tgt_names = [tgt.get_dim_name(dt, i) for i in xrange(tgt.dim(dt))]
 
-        names_in_both = obj_names & tgt_names
-
-        i = 0
-        while i < tgt.dim(dt):
-            tgt_name = tgt.get_dim_name(dt, i)
-
-            if tgt_name in names_in_both:
-                assert i < obj.dim(dt)
-
-                obj_name = obj.get_dim_name(dt, i)
-
-                if tgt_name == obj_name:
-                    i += 1
-                else:
-                    obj_name_idx, = (j for j in xrange(obj.dim(dt,i))
-                            if obj.get_dim_name(dt, j) == tgt_name)
-
-                    if i != obj_name_idx:
-                        obj = obj.move_dims(dt, i, dt, obj_name_idx, 1)
-
-                    i += 1
-            else:
-                obj = obj.insert_dims(dt, i, 1)
-                obj = obj.set_dim_name(dt, i, tgt_name)
-                i += 1
-
-        if i < obj.dim(dt) and not obj_bigger_ok:
-            raise ValueError("obj has leftover dimensions in align_spaces()")
+            obj = _align_dim_type(dt, obj, tgt, obj_bigger_ok, obj_names, tgt_names)
 
     return obj
 
 
 
 
+def align_two(obj1, obj2, across_dim_types=False):
+    """Align the spaces of two objects, potentially modifying both of them.
+
+    See also :func:`align_spaces`.
+    """
+
+    obj1 = align_spaces(obj1, obj2, obj_bigger_ok=True,
+            across_dim_types=across_dim_types)
+    obj2 = align_spaces(obj2, obj1, obj_bigger_ok=True,
+            across_dim_types=across_dim_types)
+    return (obj1, obj2)
+
+
+
+
 
 # vim: foldmethod=marker
