@@ -17,6 +17,7 @@
 #include <isl/band.h>
 #include <isl/schedule.h>
 #include <isl/flow.h>
+#include <isl/options.h>
 #include <boost/unordered_map.hpp>
 #include "gmpy.h"
 #include "wrap_helpers.hpp"
@@ -147,7 +148,9 @@ namespace isl
   };
 
   WRAP_CLASS(multi_aff);
+  WRAP_CLASS(multi_pw_aff);
   WRAP_CLASS(pw_multi_aff);
+  WRAP_CLASS(union_pw_multi_aff);
 
   WRAP_CLASS(constraint);
   WRAP_CLASS(space);
@@ -208,6 +211,7 @@ namespace isl
 
   WRAP_CLASS(access_info);
   WRAP_CLASS(flow);
+  WRAP_CLASS(restriction);
 
   inline ctx *alloc_ctx()
   {
